;===================================================================
;
; PROGRAM:  mollosvd_proj
; 
; PURPOSE:  shows 3D representation of data cube
; INPUT  :  "observed" (reduced) channel maps
; OUTPUT :  spectra/plots
; USAGE  :  This is a multi-step process
;           (1) use mollosvd_make to generate "super-channel" maps,
;           i.e.high-resolution ppv cubes. These are the raw data and
;           are generally at higher spectral (and spatial) resolution
;           than observed data sets.
;           filename: mollosvd{axis}_{emitter}_{dump#}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}.head
;           (2) these ppv-cubes can be degraded to "observations",
;           using mollosvd_trans[late]. Degrading in velocity space
;           is done by binning, degrading in position space uses 
;           a Gaussian filter.
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.head
;           The beamwidth is in pixels, and the noise level is in mJy.
; NOTES   : ilo:ihi and jlo:jhi area ALWAYS in the plane of sky.
;           Most of the RT part is from Rybicki & Lightman
;           Assumes Rayleigh-Jeans limit, i.e converts to brightness temperature
;--------------------------------------------------------------------

pro mollosvd_proj

  descr = 'Gf2'

  if (descr eq 'Gf2') then begin
    legend0= '!6Gf2'
    models = 1
    mdl    = '3aaww00d0d1_g'
    dmp    = 19
    dtdmp  = 8e-3
    convtime = 9.528e1  ;[Myr]
    idir   = 1
    lthick = [3]
    ltype  = [0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6Gf2, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    whichmol= 0
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 2 ; in pixel
    nbins    = 60l
    noise   = 0.02 ; in Kelvin
    icen    = 63l
    jcen    = 63l
    iwdt    = 63l
    subsize = 8l
    ;threshdata = 3.0
    invert=1
    contv    = 5
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/collimflow/'
    odisk    = '~/proteus/run/collimflow/plots/'
    specdisk = '~/proteus/run/collimflow/tables/'
    T0       = 1.0
    n0       = 1.0
  endif

  if (descr eq 'c3f0g3b') then begin
    legend0= '!6c3f0g3b'
    models = 3
    mdl    = 'c3f0g3b_1'
    dmp    = [25,25,25]
    dtdmp  = 2.16e-3
    n0     = 1e-2
    T0     = 1e4
    convtime = 9.528e1/sqrt(n0)  ;[Myr]
    idir   = 2
    lthick = [3,3,3]
    ltype  = [0,0,0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6c3f0g3b, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    angle  = [45,60,75]
    whichmol= [6]
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 4 ; in pixel
    nbins    = 80l
    noise   = 0.02 ; in Kelvin
    icen    = 60l
    jcen    = 120l
    iwdt    = 30l
    subsize = 15l ; this is the size of the spectrum boxes
    ;threshdata = 3.0
    invert=1
    contv    = 4
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif

  if (descr eq 'm3c0a0a_r1') then begin
    legend0= '!6Hc1b13a'
    models = 2
    mdl    = 'm3c0a0a_r1'
    dmp    = [40,40,40]
    dtdmp  = 2.16e-3
    n0     = 1e-2
    T0     = 1e4
    convtime= 9.528e1/sqrt(n0)
    idir   = 2
    lthick = [3,3,3]
    ltype  = [0,0,0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6Hc1b13a, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    angle  = [60,75]
    whichmol= [6]
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 2 ; in pixel
    nbins    = 80l
    noise   = 0.01 ; in Kelvin
    icen    = 62l
    jcen    = 220l
    iwdt    = 40l
    subsize = 15l ; this is the size of the spectrum boxes
    ;threshdata = 3.0
    invert=1
    contv    = 4
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif

  if (descr eq 'testrot') then begin
    legend0= '!6testrot'
    models = 2
    mdl    = 'testrot'
    dmp    = [0,0]
    dtdmp  = 2.16e-3
    convtime = 9.528e1/sqrt(n0)  ;[Myr]
    idir   = 2
    lthick = [3,3,3]
    ltype  = [0,0,0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6testrot, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    angle  = [45,75]
    whichmol= [6]
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 1 ; in pixel
    nbins    = 50l
    noise   = 0.02 ; in Kelvin
    icen    = 19l
    jcen    = 47l
    iwdt    = 19l
    subsize = 8l ; this is the size of the spectrum boxes
    ;threshdata = 3.0
    invert=1
    contv    = 4
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif
  ctable   = 39

;========================================================================

  convvelo = 9.118e-2*sqrt(T0)      ;[km s^(-1)]
  convtime = 9.528e1 /sqrt(n0)      ;[Myr]
  convleng = 2.74e19 *sqrt(T0/n0)   ;[cm]
  convpars = 8.896e0 *sqrt(T0/n0)   ;[pc]
  convmass = 1.72e1  *sqrt(T0^3/n0) ;[Msol]
  convsurf = 2.74e19 *sqrt(T0*n0)  
  convkmsc = 1e5
  convintb = 9d20/(2d0*1.381d-16) ; for brightness emperature

;------------------------------------------------------------------------
; the molecule "database"
  nallmol     = 6
  nmol        = n_elements(whichmol)
;  speclabel = ['!6!e12!nCO','!6!e13!nCO','!6C!e18!nO','!6NH!d3!n','!6HCO!e+!n','!6CS','!6H!d2!nCO','HCN']
  speclabel = ['!6!e12!nCO (1-0)','!6!e12!nCO (2-1)','!6!e12!nCO (3-2)',$
               '!6!e13!nCO (1-0)','!6!e13!nCO (2-1)','!6!e13!nCO (3-2)','!6HI 21cm']
  struct_12c16o_10 = {MOL, fillab:'12C16O-10'}
  struct_12c16o_21 = {MOL, fillab:'12C16O-21'}
  struct_12c16o_32 = {MOL, fillab:'12C16O-32'}
  struct_13c16o_10 = {MOL, fillab:'13C16O-10'}
  struct_13c16o_21 = {MOL, fillab:'13C16O-21'}
  struct_13c16o_32 = {MOL, fillab:'13C16O-32'}
  struct_h21cm     = {MOL, fillab:'HI-21cm'}

  molcat    = replicate(struct_12c16o_10,nallmol+1)
  molcat[1] = struct_12c16o_21
  molcat[2] = struct_12c16o_32
  molcat[3] = struct_13c16o_10
  molcat[4] = struct_13c16o_21
  molcat[5] = struct_13c16o_32
  molcat[6] = struct_h21cm

  !PATH = !PATH + ":~/lib/astrolib"

  fitsheadrd = strarr(13)

  if (nmol lt 3) then chscl = 1.0 else chscl = 2.4
  xchsz    = 1.0*chscl
  ychsz    = 1.0*chscl
  chsz     = 1.0
  chth     = 3.0

;=====================================================================
; constants
  pi     = 4d0*atan(1d0)
  clight = 3.0d10
  kboltz = 1.381d-16
  hplanck= 6.626d-27
  
  nx     = 0l
  ny     = 0l
  nv     = 0l
  nbeam  = n_elements(beamwidth)

  mintbr = 0.0
  maxtbr = 0.0

  iseed  = 151l

  if (n_elements(angle) gt 0) then isangle=1 else isangle=0

  ilo      = icen-iwdt
  ihi      = icen+iwdt
  jlo      = jcen-iwdt
  jhi      = jcen+iwdt
  nar      = float(ihi-ilo+1)*float(jhi-jlo+1)
  
  spec1arr = fltarr(2,nbins,nmol,models)

  minh1    = fltarr(nmol)
  maxh1    = fltarr(nmol)
  minh1[*] = 1e30
  maxh1[*] = -1e30
  minh2    = fltarr(nmol,models)
  maxh2    = fltarr(nmol,models)

  varr0    = findgen(nbins)
  varr     = fltarr(nbins)

  numstr = string(dmp,format='(i4.4)')
  if (isangle eq 1) then astr = '_a'+string(angle,format='(i3.3)')+'_' else astr = ''
  tmol = molcat[whichmol]
  fnm = disk+mdl+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
            +tmol.fillab+'_'+numstr+'c'+string(nbins,format='(i3.3)')+'s'$
            +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'.fits'
  print,'MOLLOSVD_SPEC: reading file ',fnm
  fitsheadrd = strarr(13)
  sredcube = ptr_new(mrdfits(fnm,0,fitsheadrd))
  print,"MOLLOSVD_SPEC: fits header:"
  print,fitsheadrd
  reads,fitsheadrd[3],nx,    format='(11x,i20)'
  reads,fitsheadrd[4],ny,    format='(11x,i20)'
  reads,fitsheadrd[5],nv,    format='(11x,i20)'
  reads,fitsheadrd[6],minvph,format='(11x,e20.5)'
  reads,fitsheadrd[7],maxvph,format='(11x,e20.5)'
  reads,fitsheadrd[8],mintbr,format='(11x,e20.5)'
  reads,fitsheadrd[9],maxtbr,format='(11x,e20.5)'
  print,format='("MOLLOSVD_SPEC: MOLECU = ",a11)',tmol.fillab
  print,format='("MOLLOSVD_SPEC: NAXIS1 = ",i5)',nx
  print,format='("MOLLOSVD_SPEC: NAXIS2 = ",i5)',ny
  print,format='("MOLLOSVD_SPEC: NAXIS3 = ",i5)',nv
  print,format='("MOLLOSVD_SPEC: MINVLS = ",e13.5)',minvph
  print,format='("MOLLOSVD_SPEC: MAXVLS = ",e13.5)',maxvph
  print,format='("MOLLOSVD_SPEC: MINTBR = ",e13.5)',mintbr
  print,format='("MOLLOSVD_SPEC: MAXTBR = ",e13.5)',maxtbr

  slicer3,sredcube

  return
end
