pro proteusread  ,filename = filename,$
                  dat      = dat,$       ; field
                  ndim     = ndim,$      ; dimension of run (1,2,3)
                  nx       = nx,$        ; active grid in x
                  ny       = ny,$        ; active grid in y
                  nz       = nz,$        ; active grid in z
                  ixbegend = ixbegend,$  ; lonarr(2), [ixbeg,ixend]
                  iybegend = iybegend,$  ; lonarr(2), [iybeg,iyend]
                  izbegend = izbegend,$  ; lonarr(2), [izbeg,izend]
                  incbdy   = incbdy,$    ; 0: no boundary layers, 1: boundary layers
                  dmpcount = dmpcount,$  ; dump number
                  timec    = timec,$     ; code time
                  iter     = iter,$      ; iteration
                  debug    = debug,$     ; gives diagnostics if set
                  swap     = swap,$      ; 0: no swap, 1: if little endian, 2: if big endian 
                  cut      = cut,$       ; ?
                  single   = single,$    ; read single precision
                  grid     = grid,$        ; read coordinate grid (grdx, grdy, grdz)
                  fits     = fits        ; use mrdfits (for large datasets)


;==============================================
; reads dumps of proteus2d.
; Can read MPI dumps or standard dumps (no distinction)
; keywords: filename : name of file to be read, without extension ".f7u"
;           dat      : data array. Will be generated if sizes do not fit.
;           nx, ny   : dimensions. If keyword set to named variable.
;           ixbeg... : start/end points. As nx...
;           cut      : for incbdy = 1 (boundary layers dumped), cuts away
;                      boundary layers. Mostly for looking at tiles from
;                      MPI-version.
;==============================================


  tndim  = 0l
  tincbdy= 0l
  tnx    = 0l
  tnxtot = 0l
  tixbeg = 0l
  tixend = 0l
  tny    = 0l
  tnytot = 0l
  tiybeg = 0l
  tiyend = 0l
  tnz    = 0l
  tnztot = 0l
  tizbeg = 0l
  tizend = 0l
  tdmpcnt= 0l
  ttimec = 0d0
  rtimec = 0.0
  titer  = 0l
;  ntmp1 = 0l
;  ntmp2 = 0l
;  ntmp3 = 0l
;  ntmp4 = 0l
;  ntmp5 = 0l
;  ntmp6 = 0l
;  ntmp7 = 0l
;  ntmp8 = 0l
;  ntmp9 = 0l
;  ntmp10= 0l
;  ntmp11= 0l
;  ntmp12= 0l
;  dtmp1 = 0d0
;  rtmp1 = 0.0

  if keyword_set(fits) then begin
    !PATH = !PATH + ':~/lib/astrolib'
    fnm = filename + '.fit'
    dat = mrdfits(fnm,0)
  endif else begin
    fnm   = filename + '.f7u'

    if keyword_set(swap) then begin
      if (swap eq 1) then begin
        openr,dun,fnm,/f77,/get_lun,/swap_if_little_endian
      endif
      if (swap eq 2) then begin
        openr,dun,fnm,/f77,/get_lun,/swap_if_big_endian
      endif
    endif else begin
      openr,dun,fnm,/f77,/get_lun
    endelse
;  readu,dun,ntmp1,ntmp2,ntmp3,ntmp4,ntmp5,ntmp6,ntmp7,ntmp8,ntmp9,ntmp10
    readu,dun,tndim,tincbdy
    readu,dun,tnx,tnxtot,tixbeg,tixend
    readu,dun,tny,tnytot,tiybeg,tiyend
    readu,dun,tnz,tnztot,tizbeg,tizend
    if keyword_set(single) then begin
      readu,dun,tdmpcnt,rtimec,titer
    endif else begin
      readu,dun,tdmpcnt,ttimec,titer
    endelse

    if keyword_set(ndim    ) then ndim      = tndim
    if keyword_set(nx      ) then nx        = tnx
    if keyword_set(ny      ) then ny        = tny
    if keyword_set(nz      ) then nz        = tnz
    if keyword_set(ixbegend) then ixbegend  = [tixbeg,tixend]
    if keyword_set(iybegend) then iybegend  = [tiybeg,tiyend]
    if keyword_set(izbegend) then iybegend  = [tizbeg,tizend]
    if keyword_set(incbdy  ) then incbdy    = tincbdy 
    if keyword_set(dmpcount) then dmpcount  = tdmpcnt
    if keyword_set(iter    ) then iter      = titer
    if keyword_set(timec   ) then begin
      if keyword_set(single) then begin
        timec     = rtimec
      endif else begin
        timec     = ttimec
      endelse
    endif

    if keyword_set(debug) then begin
      print,format='("PROTEUSREAD: file             = ",a80)', fnm
      print,format='("PROTEUSREAD: ndim             = ",i16)', tndim
      print,format='("PROTEUSREAD: incbdy           = ",i16)', tincbdy
      if (tincbdy eq 1) then begin
        print,format='("PROTEUSREAD: nxtot            = ",i16)', tnxtot
        print,format='("PROTEUSREAD: nytot            = ",i16)', tnytot
        print,format='("PROTEUSREAD: nztot            = ",i16)', tnztot
      endif else begin
        print,format='("PROTEUSREAD: nx               = ",i16)', tnx
        print,format='("PROTEUSREAD: ny               = ",i16)', tny
        print,format='("PROTEUSREAD: nz               = ",i16)', tnz
      endelse
      print,format='("PROTEUSREAD: dump             = ",i16)', tdmpcnt
      print,format='("PROTEUSREAD: iter             = ",i16)', titer
      if keyword_set(single) then begin
        print,format='("PROTEUSREAD: timec            = ",e16.9)', rtimec
      endif else begin
        print,format='("PROTEUSREAD: timec            = ",e16.9)', ttimec
      endelse
      print,format='("PROTEUSREAD: ix/y/zbg,ix/y/zed= ",i4,i4,i4,i4,i4,i4)',tixbeg,tiybeg,tizbeg,tixend,tiyend,tizend
    endif
  
;   domain is larger by 1 index for grid
;   this is for comparison with input (!) data
;   the itnx, itny, itnz are the final sizes
    if keyword_set(grid) then begin
      ttnxtot= tnxtot+1
      ttnytot= tnytot+1
      ttnztot= tnztot+1
      ttnx   = tnx+1
      ttny   = tny+1
      ttnz   = tnz+1
      tndim  = 1
    endif else begin
      ttnxtot= tnxtot
      ttnytot= tnytot
      ttnztot= tnztot
      ttnx   = tnx
      ttny   = tny
      ttnz   = tnz
    endelse
    if (tincbdy eq 1) then begin
      itnx   = ttnxtot
      itny   = ttnytot
      itnz   = ttnztot
    endif else begin
      itnx   = ttnx
      itny   = ttny
      itnz   = ttnz
    endelse
  
; check availability and sizes
    ifail = 0
    if (n_elements(dat) eq 0) then begin
      ifail = 1
    endif else begin
      if (tndim eq 1) then begin
        datsize1d = (size(dat,/dim))[0]
        if (tnx*tny eq 1) then begin
          if (itnz ne datsize1d) then ifail = 1
        endif
        if (tnx*tnz eq 1) then begin
          if (itny ne datsize1d) then ifail = 1
        endif
        if (tny*tnz eq 1) then begin
          if (itnx ne datsize1d) then ifail = 1
        endif
      endif
      if (tndim eq 2) then begin
        datsizex = (size(dat,/dim))[0]
        datsizey = (size(dat,/dim))[1]
        if (tnx eq 1) then begin
          if ((itny ne datsizex) or (itnz ne datsizey)) then ifail = 1
        endif
        if (tny eq 1) then begin
          if ((itnx ne datsizex) or (itnz ne datsizey)) then ifail = 1
        endif
        if (tnz eq 1) then begin
          if ((itnx ne datsizex) or (itny ne datsizey)) then ifail = 1
        endif
      endif
      if (tndim eq 3) then begin
        datsizex = (size(dat,/dim))[0]
        datsizey = (size(dat,/dim))[1]
        datsizez = (size(dat,/dim))[2]
        if ((itnx ne datsizex) or (itny ne datsizey) or (itnz ne datsizez)) then ifail = 1
      endif
    endelse

    if (ifail eq 1) then begin
      if (tndim eq 1) then begin
        if keyword_set(single) then dat = fltarr(max([itnx,itny,itnz])) else dat = dblarr(max([itnx,itny,itnz]))
      endif else begin
        if (tndim eq 3) then begin
          if keyword_set(single) then dat = fltarr(itnx,itny,itnz) else dat = dblarr(itnx,itny,itnz)
        endif else begin
          if (tnx eq 1) then begin
            if keyword_set(single) then dat = fltarr(itny,itnz) else dat = dblarr(itny,itnz)
          endif    
          if (tny eq 1) then begin
            if keyword_set(single) then dat = fltarr(itnx,itnz) else dat = dblarr(itnx,itnz)
          endif
          if (tnz eq 1) then begin
            if keyword_set(single) then dat = fltarr(itnx,itny) else dat = dblarr(itnx,itny)
          endif
        endelse
      endelse
    endif
    readu,dun,dat
    free_lun,dun
  endelse

  if keyword_set(debug) then begin
    print,format='("PROTEUSREAD: min/max(arr)       = ",e13.5,e13.5)',min(dat),max(dat)
  endif

;---------------------------------

  return
end

